@@ -1,6 +1,6 @@
 
 /*************************************************************************
- * Copyright (C) 2007-2008 Ruben Pollan Bella <meskio@amedias.org>       *
+ * Copyright (C) 2007-2009 Ruben Pollan Bella <meskio@sindominio.net>    *
  *                                                                       *
  *  This file is part of TuDu.                                           *
  *                                                                       *
@@ -289,7 +289,7 @@ void Config::getThemeRow(string& value)
 
 	if (row_index == MAX_THEME_ROWS)
 	{
-		fprintf(stderr, "Error: too much rows in theme\n");
+		fprintf(stderr, "Error: too many rows in theme\n");
 		exit(1);
 	}
 
@@ -325,7 +325,7 @@ void Config::getThemeRow(string& value)
 		i=value.find(",", j);
 		if (win == MAX_THEME_WINDOWS)
 		{
-			fprintf(stderr, "Error: too much windows in a row for theme\n");
+			fprintf(stderr, "Error: too many windows in a row for theme\n");
 			exit(1);
 		}
 
@@ -400,7 +400,7 @@ void Config::getThemeTree(string& value)
 	{
 		if (tree_index>MAX_THEME_TREECOLS)
 		{
-			fprintf(stderr, "Error: too much cols in a tree for theme\n");
+			fprintf(stderr, "Error: too many cols in a tree for theme\n");
 			exit(1);
 		}
 		i=value.find(",", j);
@@ -581,7 +581,23 @@ char* Config::getEditor()
 	return editor;
 }
 
-bool Config::genWindowCoor(int lines, int cols, window_coor coor[])
+void Config::genWindowCoor(int lines, int cols, window_coor coor[])
+{
+	/* if window don't fits will display only the tree */
+	if (!_genWindowCoor(lines, cols, coor))
+	{
+		for (int i = 0; i < NUM_WINDOWS; ++i)
+			coor[i].exist = false;
+
+		coor[WTREE].exist = true;
+		coor[WTREE].x = 0;
+		coor[WTREE].y = 0;
+		coor[WTREE].lines = lines;
+		coor[WTREE].cols = cols;
+	}
+}
+
+bool Config::_genWindowCoor(int lines, int cols, window_coor coor[])
 {
 	int height[MAX_THEME_ROWS];
 	int undefined_row;
@@ -589,6 +605,9 @@ bool Config::genWindowCoor(int lines, int cols, window_coor coor[])
 	int y = 0,x = 0;
 	bool undefined_height = false;
 
+	/* check if there is enought rows */
+	if (lines < row_index + 8) return false;
+
 	/* initialice the windows as not present */
 	for (int i=0; i<NUM_WINDOWS; ++i) coor[i].exist = false;
 
