@@ -1,6 +1,6 @@
 
 /**************************************************************************
- * Copyright (C) 2007-2008 Ruben Pollan Bella <meskio@amedias.org>        *
+ * Copyright (C) 2007-2009 Ruben Pollan Bella <meskio@sindominio.net>     *
  *                                                                        *
  *  This file is part of TuDu.                                            *
  *                                                                        *
@@ -19,6 +19,8 @@
 
 #include "data.h"
 
+set<string> categories;
+
 ToDo::ToDo(string t, bool isDone, string txt) 
 		: title(t), _done(isDone)
 {
@@ -74,11 +76,18 @@ int& ToDo::priority()
 	return _priority;
 }
 
-string& ToDo::category()
+string ToDo::getCategory()
 {
 	return _category;
 }
 
+void ToDo::setCategory(string c)
+{
+	_category = c;
+
+	if (c != NONE_CATEGORY) categories.insert(c);
+}
+
 bool ToDo::haveChild()
 {
 	return !childs.empty();
@@ -314,6 +323,11 @@ iToDo& iToDo::operator=(const iToDo& t)
 	return *this;
 }
 
+bool iToDo::operator==(const iToDo& t)
+{
+	return (*t.cursor) == (*cursor);
+}
+
 bool iToDo::operator++()
 {
 	if (!end())
@@ -370,6 +384,54 @@ bool iToDo::out()
 	}
 }
 
+void iToDo::next()
+{
+	in();
+	/* if there is no childs */
+	if (end())
+	{
+		/* while is the end of the childs go to the parent and move forward */
+		while (end() && !path.empty()) {
+			out();
+			operator++();
+		}
+
+		/* if is the last task continue from the beginning */
+		if (end() && path.empty())
+			cursor = parent->childs.begin();
+	}
+}
+
+void iToDo::prev()
+{
+	/* if is not the first child move up and go as deep as possible */
+	if (!begin())
+	{
+		operator--();
+		while ((*cursor)->haveChild())
+		{
+			in();
+			while (operator++());
+			operator--();
+		}
+	}
+	/* if is the first child go to the parent */
+	else
+	{
+		if (!out())
+		{
+			cursor = parent->childs.end();
+			cursor--;
+			while ((*cursor)->haveChild())
+			{
+				in();
+				while (operator++());
+				operator--();
+			}
+		}
+	}
+}
+
 ToDo &iToDo::operator*()
 {
 	if (!end()) return *(*cursor);
@@ -474,3 +536,35 @@ void iToDo::sort(char order[])
 	}
 	root->_sort();
 }
+
+bool iToDo::search(string& str)
+{
+	pToDo begin = *cursor;
+
+	while (1)
+	{
+		next();
+		/* if the title contains str */
+		if ((*cursor)->title.find(str, 0) != string::npos)
+			return true;
+		/* if is the same as the beginning */
+		if (*cursor == begin)
+			return false;
+	}
+}
+
+bool iToDo::searchUp(string& str)
+{
+	pToDo begin = *cursor;
+
+	while (1)
+	{
+		prev();
+		/* if the title contains str */
+		if ((*cursor)->title.find(str, 0) != string::npos)
+			return true;
+		/* if is the same as the beginning */
+		if (*cursor == begin)
+			return false;
+	}
+}
