@@ -1,6 +1,6 @@
 
 /**************************************************************************
- * Copyright (C) 2007-2008 Ruben Pollan Bella <meskio@amedias.org>        *
+ * Copyright (C) 2007-2009 Ruben Pollan Bella <meskio@sindominio.net>     *
  *                                                                        *
  *  This file is part of TuDu.                                            *
  *                                                                        *
@@ -25,10 +25,14 @@
 #include <stack>
 #include <string>
 #include <algorithm>
+#include <set>
 #include "text.h"
 #include "date.h"
 using namespace std;
 
+extern set<string> categories;
+#define NONE_CATEGORY "NONE"
+
 struct child_info {
 	int percent;
 	Date deadline;
@@ -58,7 +62,8 @@ public:
 	int& schedPosition();
 	bool& done();
 	int& priority();
-	string& category();
+	string getCategory();
+	void setCategory(string c);
 	bool haveChild();
 	child_info getChildInfo();
 	int line;
@@ -93,10 +98,13 @@ public:
 	iToDo(ToDo& t);
 
 	iToDo& operator=(const iToDo& t);
+	bool operator==(const iToDo& t);
 	bool operator++();
 	bool operator--();
 	bool in();
 	bool out();
+	void next();
+	void prev();
 	ToDo &operator*();
 	ToDo *operator->();
 	bool end();
@@ -118,6 +126,8 @@ public:
 	 * the same leters in capital are for inverse order
 	 */
 	void sort(char order[]);
+	bool search(string& str);
+	bool searchUp(string& str);
 protected:
 	pToDo root;
 	stack<list<pToDo>::iterator> path;
