@@ -1,6 +1,6 @@
 
 /**************************************************************************
- * Copyright (C) 2007-2008 Ruben Pollan Bella <meskio@amedias.org>        *
+ * Copyright (C) 2007-2009 Ruben Pollan Bella <meskio@sindominio.net>     *
  *                                                                        *
  *  This file is part of TuDu.                                            *
  *                                                                        *
@@ -21,6 +21,11 @@
 #define EDITOR_H
 
 #include <string>
+#include <string.h>
+#include <list>
+#include <set>
+#include <map>
+#include <vector>
 #include "window.h"
 
 class Editor
@@ -36,15 +41,21 @@ protected:
 	int cursor;
 	int key;
 	bool exit;
+	bool result;
 
+	virtual void initialize();
 	virtual void left();
 	virtual void right();
 	virtual void up();
 	virtual void down();
+	virtual void home();
+	virtual void end();
 	virtual void backspace();
 	virtual void supr();
-	virtual void esc();
+	virtual void tab();
 	virtual void other();
+	virtual void enter();
+	virtual void esc();
 };
 
 class LineEditor: public Editor
@@ -52,18 +63,60 @@ class LineEditor: public Editor
 protected:
 	void left();
 	void right();
+	void home();
+	void end();
 	void backspace();
 	void supr();
-	void esc();
+	void tab();
 	void other();
 };
 
+class CategoryEditor: public LineEditor
+{
+protected:
+	set<string>::iterator search;
+	set<string>::iterator first;
+	int length;
+
+	void initialize();
+	void tab();
+};
+
+class HistoryEditor: public LineEditor
+{
+protected:
+	list<string> history;
+	list<string>::iterator shown;
+
+	void initialize();
+	void up();
+	void down();
+	void enter();
+};
+
+class CmdEditor: public HistoryEditor
+{
+protected:
+	map<string,string>::iterator com_search;
+	map<string,string>::iterator com_first;
+	set<string>::iterator search;
+	set<string>::iterator first;
+	int length;
+	int param;
+
+	void initialize();
+	void tab();
+	void command_completion(string& com);
+	void category_completion(string& cat, int num_param);
+};
+
 class DateEditor: public Editor
 {
 protected:
 	void left();
 	void right();
-	void esc();
+	void home();
+	void end();
 	void other();
 };
 
@@ -74,7 +127,6 @@ protected:
 	void down();
 	void backspace();
 	void supr();
-	void esc();
 	void other();
 };
 
