@@ -1,6 +1,6 @@
 
 /**************************************************************************
- * Copyright (C) 2007-2008 Ruben Pollan Bella <meskio@amedias.org>        *
+ * Copyright (C) 2007-2009 Ruben Pollan Bella <meskio@sindominio.net>     *
  *                                                                        *
  *  This file is part of TuDu.                                            *
  *                                                                        *
@@ -22,9 +22,12 @@
 #define cursor_line()  (cursor->line-tree_begin)
 #define isCollapse() ((cursor->getCollapse()) && (!cursor->actCollapse()))
 
-Interface::Interface(Screen &s, iToDo &t, Sched& sch, Config &c, Writer &w) 
-		: screen(s), cursor(t), sched(sch), config(c), writer(w), copied(NULL)
+Interface::Interface(Screen &s, iToDo &t, Sched& sch, Config &c, Writer &w, Cmd &com) 
+		: screen(s), cursor(t), sched(sch), config(c), writer(w), copied(NULL), cmd(com)
 {
+	cmd.get_interface(this);
+
+	search_pattern = "";
 	tree_begin = 0;
 	tree_end = screen.treeLines();
 
@@ -93,6 +96,10 @@ void Interface::main()
 			if ("upText" == action) upText();
 			if ("collapse" == action) collapse();
 			if ("hideDone" == action) hide_done();
+			if ("search" == action) search();
+			if ("searchNext" == action) search_next();
+			if ("searchPrev" == action) search_prev();
+			if ("cmd" == action) command_line();
 			if ("sortByTitle" == action) sortByTitle();
 			if ("sortByDone" == action) sortByDone();
 			if ("sortByDeadline" == action) sortByDeadline();
@@ -201,7 +208,29 @@ void Interface::eraseCursor()
 
 void Interface::drawCursor()
 {
-	if (cursor->line > tree_end - 4)
+	if (screen.treeLines() < 8)
+	{
+		if (cursor->line >= tree_end)
+		{
+			tree_end = cursor->line;
+			tree_begin = tree_end - screen.treeLines();
+			drawTodo();
+		}
+		else if (cursor->line < tree_begin)
+		{
+			tree_begin = cursor->line;
+			tree_end = tree_begin + screen.treeLines();
+			drawTodo();
+		}
+		else
+		{
+			screen.drawTask(cursor_line(), cursor.depth(), *cursor, 
+					true);
+			screen.drawText(cursor->getText());
+			screen.drawSched(sched, &(*cursor));
+		}
+	}
+	else if (cursor->line > tree_end - 4)
 	{
 		tree_end = cursor->line + 4;
 		tree_begin = tree_end - screen.treeLines();
@@ -227,7 +256,34 @@ void Interface::drawCursor()
 
 bool Interface::isHide(iToDo& todo)
 {
-	return (config.getHideDone() && todo->done());
+	bool hide;
+
+	/* if is done */
+	hide = (config.getHideDone() && todo->done());
+	/* if is in hidden category */
+	hide = hide || (hidden_categories.count(todo->getCategory()));
+
+	return hide;
+}
+
+void Interface::newTodo()
+{
+	cursor.addChild(new ToDo());
+
+	/* inherit category */
+	if (cursor.depth())
+	{
+		iToDo father = cursor;
+		father.out();
+		cursor->setCategory(father->getCategory());
+	}
+	else if (hidden_categories.count(""))
+	{
+		set<string>::iterator cat;
+		for (cat = categories.begin(); (cat != categories.end()) && hidden_categories.count(*cat); cat++);
+		if (cat != categories.end()) cursor->setCategory(*cat);
+		else cursor->setCategory(NONE_CATEGORY);
+	}
 }
 
 void Interface::left()
@@ -238,7 +294,7 @@ void Interface::left()
 		up();
 	}
 	else if (isHide(cursor))
-		cursor.addChild(new ToDo());
+		newTodo();
 	cursor->actCollapse() = false;
 	if (cursor->getCollapse())
 	{
@@ -261,7 +317,7 @@ void Interface::right()
 
 	if (cursor.end())
 	{
-		cursor.addChild(new ToDo());
+		newTodo();
 		drawTodo();
 		string title;
 		if ((editLine(title)) && (title != ""))
@@ -445,7 +501,7 @@ bool Interface::editLine(string& str)
 	bool save;
 
 	str = cursor->getTitle();
-	screen.infoMsg("Editing todo. Press enter for save it or esc for not save");
+	screen.infoMsg("Editing todo. Press ENTER to save or ESC to abort edit");
 	save = screen.editTitle(cursor_line(), cursor.depth(), 
 			cursor->haveChild(), str);
 	screen.infoClear();
@@ -454,7 +510,7 @@ bool Interface::editLine(string& str)
 
 void Interface::editDeadline()
 {
-	screen.infoMsg("Editing deadline. Press enter for save it or esc for not save");
+	screen.infoMsg("Editing deadline. Press ENTER to save or ESC to abort edit");
 	screen.editDeadline(cursor_line(), cursor->deadline(), cursor->done());
 	screen.infoClear();
 	drawTodo();
@@ -462,7 +518,7 @@ void Interface::editDeadline()
 
 void Interface::setPriority()
 {
-	screen.infoMsg("Editing priority. Press enter for save it or esc for not save");
+	screen.infoMsg("Editing priority. Press ENTER to save or ESC to abort edit");
 	screen.setPriority(cursor_line(), cursor->priority());
 	screen.infoClear();
 	drawTodo();
@@ -470,16 +526,15 @@ void Interface::setPriority()
 
 void Interface::setCategory()
 {
-	screen.infoMsg("Editing category. Press enter for save it or esc for not save");
-	screen.setCategory(cursor_line(), cursor->category());
+	screen.infoMsg("Editing category. Press ENTER to save or ESC to abort edit");
+	screen.setCategory(cursor_line(), *cursor);
 	screen.infoClear();
 	drawTodo();
 }
 
 void Interface::addLine()
 {
-
-	cursor.addChild(new ToDo());
+	newTodo();
 	drawTodo();
 	string title;
 	if ((editLine(title)) && (title != ""))
@@ -490,7 +545,7 @@ void Interface::addLine()
 
 void Interface::addLineUp()
 {
-	cursor.addChildUp(new ToDo());
+	newTodo();
 	drawTodo();
 	string title;
 	if ((editLine(title)) && (title != ""))
@@ -514,7 +569,6 @@ void Interface::editText()
 		char path[L_tmpnam];
 		char s[86];
 		char* argv[32];
-		\
 		int argc;
 		char* res;
 		string str;
@@ -532,7 +586,6 @@ void Interface::editText()
 		ofs.close();
 		sprintf(s, editor, path);
 
-		screen.infoMsg(s);
 		argc=0;
 		argv[argc] = strtok(s, " \t");
 		while ((res = strtok(NULL, " \t")) != NULL)
@@ -542,6 +595,7 @@ void Interface::editText()
 		}
 		argv[argc+1] = NULL;
 		
+		endwin();
 		switch (fork())
 		{
 			case 0:
@@ -554,6 +608,7 @@ void Interface::editText()
 				wait(NULL);
 				break;
 		}
+		doupdate();
 		
 		ifstream ifs(path);
 		char c;
@@ -568,13 +623,10 @@ void Interface::editText()
 		screen.drawText(cursor->getText());
 		noecho();
 		curs_set(0);
-		
-		/* force redraw screen, because it don't recover well sometimes */
-		resizeTerm();
 	}
 	else
 	{
-		screen.infoMsg("Editing text. Press esc for save it");
+		screen.infoMsg("Editing text. Press ESC to save");
 		screen.editText(cursor->getText());
 		screen.infoClear();
 	}
@@ -621,11 +673,95 @@ void Interface::hide_done()
 	config.getHideDone() = !config.getHideDone();
 	if (isHide(cursor)) up();
 	if (!config.getHideDone() && cursor->getTitle().empty())
-		//FIXME: if the user have an title empty will be destroy
 		del();
 	drawTodo();
 }
 
+bool Interface::_search()
+{
+	bool res = true;
+	iToDo hit = cursor;
+
+	if (hit.search(search_pattern))
+	{
+		iToDo first_hit = hit;
+		/* if is hide search the next one */
+		while (isHide(hit))
+		{
+			hit.search(search_pattern);
+			/* if there is only hidden tasks exit */
+			if (first_hit == hit)
+			{
+				res = false;
+				break;
+			}
+		}
+	}
+	else
+		res = false;
+
+	if (res)
+	{
+		while (cursor.out()) cursor->actCollapse() = false;
+		cursor = hit;
+		while (hit.out()) hit->actCollapse() = true;
+	}
+	return res;
+}
+
+void Interface::command_line()
+{
+	string command("");
+
+	if (screen.cmd(command))
+	{
+		if (cmd.cmd(command))
+		{
+			if (isHide(cursor)) up();
+			/* destroy empty with NONE category */
+			if ((cursor->getCategory() == NONE_CATEGORY) && cursor->getTitle().empty())
+					del();
+			drawTodo();
+		}
+	}
+}
+
+void Interface::search()
+{
+	string pattern("");
+
+	if (screen.searchText(pattern))
+	{
+		search_pattern = pattern;
+		if (_search())
+			drawTodo();
+		else
+			screen.infoMsg("Not found");
+	}
+}
+
+void Interface::search_next()
+{
+	if (search_pattern != "")
+		if (_search())
+			drawTodo();
+		else
+			screen.infoMsg("Not found");
+	else
+		screen.infoMsg("No search pattern");
+}
+
+void Interface::search_prev()
+{
+	if (search_pattern != "")
+		if (_search())
+			drawTodo();
+		else
+			screen.infoMsg("Not found");
+	else
+		screen.infoMsg("No search pattern");
+}
+
 void Interface::sortByTitle()
 {
 	sortOrder[0] = 't';
@@ -700,12 +836,18 @@ void Interface::sortRevUser()
 
 void Interface::save()
 {
-	writer.save();
-	cursor.sort(sortOrder);
-	screen.infoMsg("File saved");
+	if (writer.save())
+	{
+		cursor.sort(sortOrder);
+		screen.infoMsg("File saved");
+	}
+	else
+	{
+		screen.infoMsg("File can not be saved");
+	}
 }
 
-#define LINES_HELP 44
+#define LINES_HELP 34
 void Interface::help()
 {
 	action_list list;
@@ -725,38 +867,29 @@ void Interface::help()
 	str[i++] = "  " + list["delDeadline"] + "\tdelete deadline\n";
 	str[i++] = "  " + list["delPriority"] + "\tdelete priority\n";
 	str[i++] = "  " + list["delSched"] + "\tdelete schedule\n";
-	str[i++] = "  " + list["paste"] + "\tpaste the last deleted\n";
-	str[i++] = "  " + list["pasteUp"] + "\tpaste the last deleted upper than the cursor\n";
-	str[i++] = "  " + list["pasteChild"] + "\tpaste the last deleted as child of the task\n";
+	str[i++] = "  " + list["paste"] + "\tpaste the last deletion\n";
+	str[i++] = "  " + list["pasteUp"] + "\tpaste the last deletion above the cursor\n";
+	str[i++] = "  " + list["pasteChild"] + "\tpaste the last deletion as child of the task\n";
 	str[i++] = "  " + list["addTodo"] + "\tadd line\n";
 	str[i++] = "  " + list["addTodoUp"] + "\tadd line upper than the cursor\n";
 	str[i++] = "  " + list["editTitle"] + "\tmodify line\n";
 	str[i++] = "  " + list["editText"] + "\tedit text\n";
 	str[i++] = "  " + list["editDeadline"] + "\tedit/add deadline\n";
 	str[i++] = "  " + list["editSched"] + "\tedit/add schedule\n";
-	str[i++] = "  " + list["setPriority"] + "\tadd or modify the priority\n";
-	str[i++] = "  " + list["setCategory"] + "\tadd or modify the category\n";
-	str[i++] = "  " + list["downText"] + "\tscroll down the text\n";
-	str[i++] = "  " + list["upText"] + "\tscroll up the text\n";
+	str[i++] = "  " + list["setPriority"] + "\tadd or modify priority\n";
+	str[i++] = "  " + list["setCategory"] + "\tadd or modify category\n";
+	str[i++] = "  " + list["downText"] + "\tscroll text down\n";
+	str[i++] = "  " + list["upText"] + "\tscroll text up\n";
 	str[i++] = "  " + list["schedUp"] + "\tmove up the task on the scheduler\n";
 	str[i++] = "  " + list["schedDown"] +  "\tmove down the task on the scheduler\n";
-	str[i++] = "  " + list["collapse"] + "\tcollapse childs\n";
-	str[i++] = "  " + list["hideDone"] + "\thide done tasks\n";
-	str[i++] = "  " + list["sortByTitle"] + "\tsort todo by title\n";
-	str[i++] = "  " + list["sortByDone"] + "\tsort todo by done\n";
-	str[i++] = "  " + list["sortByDeadline"] + "\tsort todo by deadline\n";
-	str[i++] = "  " + list["sortByPriority"] + "\tsort todo by priority\n";
-	str[i++] = "  " + list["sortByCategory"] + "\tsort todo by category\n";
-	str[i++] = "  " + list["sortByUser"] + "\tsort todo by user order\n";
-	str[i++] = "  " + list["sortRevTitle"] + "\tsort todo by title in inverse order\n";
-	str[i++] = "  " + list["sortRevDone"] + "\tsort todo by done in inverse order\n";
-	str[i++] = "  " + list["sortRevDeadline"] + "\tsort todo by deadline in inverse order\n";
-	str[i++] = "  " + list["sortRevPriority"] + "\tsort todo by priority in inverse order\n";
-	str[i++] = "  " + list["sortRevCategory"] + "\tsort todo by category in inverse order\n";
-	str[i++] = "  " + list["sortRevUser"] + "\tsort todo by user in inverse order\n";
+	str[i++] = "  " + list["collapse"] + "\tcollapse children\n";
+	str[i++] = "  " + list["hideDone"] + "\thide tasks that are done\n";
+	str[i++] = "  " + list["search"] + "\tsearch on titles\n";
+	str[i++] = "  " + list["searchNext"] + "\tgo to next search result\n";
+	str[i++] = "  " + list["cmd"] + "\topen command input\n";
 	str[i++] = "  " + list["save"] + "\tsave todo\n";
-	str[i++] = "  " + list["help"] + "\tshow help\n";
 	str[i++] = "  " + list["quit"] + "\tquit\n";
 	str[i] = "  " + list["quitNoSave"] + "\tquit without save\n";
 	screen.helpPopUp(str, i);
 }
+
