@@ -1,6 +1,6 @@
 
 /**************************************************************************
- * Copyright (C) 2007-2008 Ruben Pollan Bella <meskio@amedias.org>        *
+ * Copyright (C) 2007-2009 Ruben Pollan Bella <meskio@sindominio.net>     *
  *                                                                        *
  *  This file is part of TuDu.                                            *
  *                                                                        *
@@ -32,21 +32,27 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <map>
+#include <set>
 #include <fstream>
+
+class Interface;
 #include "data.h"
 #include "sched.h"
 #include "config.h"
 #include "parser.h"
 #include "editor.h"
 #include "screen.h"
+#include "cmd.h"
+
 
 class Interface
 {
 public:
-	Interface(Screen& s, iToDo &t, Sched& sch, Config &c, Writer &w);
+	Interface(Screen& s, iToDo &t, Sched& sch, Config &c, Writer &w, Cmd &com);
 	~Interface();
 
 	void main();
+	friend class Cmd;
 private:
 	Screen &screen;
 	iToDo &cursor;
@@ -57,6 +63,9 @@ private:
 	int tree_begin, tree_end;
 	pToDo copied;
 	char sortOrder[16];
+	string search_pattern;
+	Cmd &cmd; /* command interface */
+	set<string> hidden_categories;
 
 	void resizeTerm();
 	void drawTodo();
@@ -65,6 +74,7 @@ private:
 	void eraseCursor();
 	void drawCursor();
 	bool isHide(iToDo& todo);
+	void newTodo();
 	void left();
 	void right();
 	void up();
@@ -94,6 +104,11 @@ private:
 	void downText();
 	void collapse();
 	void hide_done();
+	void command_line();
+	bool _search();
+	void search();
+	void search_next();
+	void search_prev();
 	void sortByTitle();
 	void sortByDone();
 	void sortByDeadline();
