@@ -1,6 +1,6 @@
 
 /**************************************************************************
- * Copyright (C) 2007-2008 Ruben Pollan Bella <meskio@amedias.org>        *
+ * Copyright (C) 2007-2009 Ruben Pollan Bella <meskio@sindominio.net>     *
  *                                                                        *
  *  This file is part of TuDu.                                            *
  *                                                                        *
@@ -198,7 +198,7 @@ void Parser::ptag(iToDo& iterator, Sched& sched)
 	}
 	if ("/category" == str)
 	{
-		iterator->category() = txt;
+		iterator->setCategory(txt);
 	}
 	if ("text" == str)
 	{
@@ -249,18 +249,25 @@ Writer::Writer(const char* pathToSave, ToDo& t): todo(t)
 
 Writer::~Writer() {}
 
-void Writer::save()
+bool Writer::save()
 {
 	file.open(path);
 	file << "<?xml version=\"1.0\"?>" << endl;
+
+	/* there was an error writing */
+	if (file.fail()) return false;
+
 	file << "<!DOCTYPE tudu SYSTEM \"" << PATH_DTD << "\">" << endl;
 	file << "<todo>" << endl;
 	i = new iToDo(todo);
 	i->sort((char*)"");
 	while(--(*i));
 	_save();
+	delete i;
 	file << "</todo>" << endl;
 	file.close();
+
+	return true;
 }
 
 #define putTabs(num) \
@@ -308,10 +315,10 @@ void Writer::_save()
 			file << "<priority>" << (*i)->priority() << 
 					"</priority>" << endl;
 		}
-		if (!(*i)->category().empty())
+		if (!(*i)->getCategory().empty())
 		{
 			putTabs((*i).depth()+1);
-			file << "<category>" << (*i)->category() << 
+			file << "<category>" << (*i)->getCategory() << 
 					"</category>" << endl;
 		}
 		if ((*i)->getText() != "")
