@@ -1,6 +1,6 @@
 
 /**************************************************************************
- * Copyright (C) 2007-2008 Ruben Pollan Bella <meskio@amedias.org>        *
+ * Copyright (C) 2007-2009 Ruben Pollan Bella <meskio@sindominio.net>     *
  *                                                                        *
  *  This file is part of TuDu.                                            *
  *                                                                        *
@@ -54,10 +54,10 @@ private:
 class Writer
 {
 public:
-	Writer(const char* path, ToDo& t);
+	Writer(const char* pathToSave, ToDo& t);
 	~Writer();
 
-	void save();
+	bool save(); /* return if was succesfull */
 private:
 	ofstream file;
 	ToDo& todo;
