@@ -1,6 +1,6 @@
 
 /*************************************************************************
- * Copyright (C) 2007-2008 Ruben Pollan Bella <meskio@amedias.org>       *
+ * Copyright (C) 2007-2009 Ruben Pollan Bella <meskio@sindominio.net>    *
  *                                                                       *
  *  This file is part of TuDu.                                           *
  *                                                                       *
@@ -98,15 +98,8 @@ void Screen::draw()
 	int lines, cols;
 
 	/* get size of windows */
-        getmaxyx(stdscr, lines, cols); 
-	if (!config.genWindowCoor(lines, cols, coor))
-	{
-		clear();
-		refresh();
-		endwin();
-		fprintf(stderr, "Err: Theme don't fits\n");
-		exit(1);
-	}
+    getmaxyx(stdscr, lines, cols); 
+	config.genWindowCoor(lines, cols, coor);
 
 	/* create windows */
 	window_coor c = coor[WTREE];
@@ -166,20 +159,18 @@ void Screen::resizeTerm()
 	endwin();
 	refresh();
 	getmaxyx(stdscr, lines, cols);
-	if (!config.genWindowCoor(lines, cols, coor))
-	{
-		clear();
-		refresh();
-		endwin();
-		fprintf(stderr, "Err: Theme don't fits\n");
-		exit(1);
-	}
+	config.genWindowCoor(lines, cols, coor);
 
-	/* resize windows */
+	/* 
+	 * resize windows
+	 */
+	/* tree */
 	window_coor c = coor[WTREE];
 	wtree->_resize(c.lines, c.cols);
 	wtree->_mv(c.y, c.x);
 	wtree->_erase();
+
+	/* help */
 	c = coor[WHELP];
 	if (c.exist)
 	{
@@ -196,6 +187,8 @@ void Screen::resizeTerm()
 		whelp->_refresh();
 		whelp->_attroff(COLOR_HELP);
 	}
+
+	/* priority */
 	c = coor[WPRIORITY];
 	if (c.exist)
 	{
@@ -203,6 +196,8 @@ void Screen::resizeTerm()
 		wpriority->_mv(c.y, c.x);
 		wpriority->_erase();
 	}
+
+	/* category */
 	c = coor[WCATEGORY];
 	if (c.exist)
 	{
@@ -210,6 +205,8 @@ void Screen::resizeTerm()
 		wcategory->_mv(c.y, c.x);
 		wcategory->_erase();
 	}
+
+	/* deadline */
 	c = coor[WDEADLINE];
 	if (c.exist)
 	{
@@ -217,6 +214,8 @@ void Screen::resizeTerm()
 		wdeadline->_mv(c.y, c.x);
 		wdeadline->_erase();
 	}
+
+	/* deadline */
 	c = coor[WTEXT];
 	if (c.exist)
 	{
@@ -224,6 +223,8 @@ void Screen::resizeTerm()
 		wtext->_mv(c.y, c.x);
 		wtext->_erase();
 	}
+
+	/* info */
 	c = coor[WINFO];
 	if (c.exist)
 	{
@@ -231,6 +232,8 @@ void Screen::resizeTerm()
 		winfo->_mv(c.y, c.x);
 		winfo->_erase();
 	}
+
+	/* schedule */
 	c = coor[WSCHEDULE];
 	if (c.exist)
 	{
@@ -321,7 +324,7 @@ void Screen::drawTask(int line, int depth, ToDo& t, bool isCursor)
 	/* draw category */
 	if (coor[WCATEGORY].exist)
 	{
-		string& category = t.category();
+		string category = t.getCategory();
 		if (isCursor)
 			wcategory->_attron(COLOR_SELECTED);
 		else
@@ -654,15 +657,17 @@ void Screen::setPriority(int line, int& priority)
 	}
 }
 
-void Screen::setCategory(int line, string& category)
+void Screen::setCategory(int line, ToDo& t)
 {
 	if (coor[WCATEGORY].exist)
 	{
-		lineEditor.getText() = category;
-		lineEditor.cursorPos() = category.length();
-		if (lineEditor.edit(*wcategory, line, 0, CATEGORY_LENGTH))
+		string category = t.getCategory();
+		categoryEditor.getText() = category;
+		categoryEditor.cursorPos() = category.length();
+		if (categoryEditor.edit(*wcategory, line, 0, CATEGORY_LENGTH))
 		{
-			category = lineEditor.getText();
+			category = categoryEditor.getText();
+			t.setCategory(category);
 		}
 
 		wcategory->_move(line, 0);
@@ -706,6 +711,46 @@ int Screen::treeLines()
 	return wtree->_lines();
 }
 
+bool Screen::searchText(string& pattern)
+{
+	if (coor[WINFO].exist)
+	{
+		bool save;
+
+		infoClear();
+		winfo->_addch(0,0,'/');
+		save = searchEditor.edit(*winfo, 0, 1, 
+				PERCENT_COL-2);
+		pattern = searchEditor.getText();
+		infoClear();
+		return save;
+	}
+	else
+	{
+		return false;
+	}
+}
+
+bool Screen::cmd(string& command)
+{
+	if (coor[WINFO].exist)
+	{
+		bool save;
+
+		infoClear();
+		winfo->_addch(0,0,':');
+		save = cmdEditor.edit(*winfo, 0, 1, 
+				PERCENT_COL-2);
+		command = cmdEditor.getText();
+		infoClear();
+		return save;
+	}
+	else
+	{
+		return false;
+	}
+}
+
 void Screen::infoMsg(const char str[])
 {
 	if (coor[WINFO].exist)
@@ -750,7 +795,7 @@ void Screen::infoPercent(int percent)
 	do { \
 	h._erase(); \
 	h._move(0,0); \
-	for (int i = cursor; (i<len) && (i-cursor<lines); ++i) \
+	for (int i = cursor; (i<=len) && (i-cursor<lines); ++i) \
 		h._addstr(str[i]); \
 	h._refresh(); \
 	} while (0)
@@ -760,7 +805,7 @@ void Screen::helpPopUp(string str[], int len)
         getmaxyx(stdscr, lines, cols); 
 	Window help_box(lines-8, cols-16, 4, 8);
 	help_box._box();
-	string s = "--- Pulse any key for close ---";
+	string s = "--- Press any key to close ---";
 	help_box._move(lines-10, (cols/2)-8-(s.length()/2));
 	help_box._addstr(s);
 	help_box._refresh();
@@ -780,7 +825,7 @@ void Screen::helpPopUp(string str[], int len)
 				resized = true;
 				break;
 			case KEY_DOWN:
-				if (cursor < len-lines)
+				if (cursor <= len-lines)
 				{
 					++cursor;
 					draw_help();
@@ -793,12 +838,24 @@ void Screen::helpPopUp(string str[], int len)
 					draw_help();
 				}
 				break;
+			case  KEY_NPAGE:
+				cursor += lines;
+				if (cursor > len-lines)
+					cursor = len + 1 - lines;
+				draw_help();
+				break;
+			case  KEY_PPAGE:
+				cursor -= lines;
+				if (cursor < 0)
+					cursor = 0;
+				draw_help();
+				break;
 			case KEY_HOME:
 				cursor = 0;
 				draw_help();
 				break;
 			case KEY_END:
-				cursor = len - lines;
+				cursor = len + 1 - lines;
 				draw_help();
 				break;
 			default: 
@@ -810,6 +867,7 @@ void Screen::helpPopUp(string str[], int len)
 	if (resized) ungetch(KEY_RESIZE);
 	h._erase();
 
+	/* redraw everything, removing the help window */
 	redrawwin(stdscr);
 	refresh();
 	wtree->_redraw();
