@@ -1,6 +1,6 @@
 
 /**************************************************************************
- * Copyright (C) 2007-2008 Ruben Pollan Bella <meskio@amedias.org>        *
+ * Copyright (C) 2007-2009 Ruben Pollan Bella <meskio@sindominio.net>     *
  *                                                                        *
  *  This file is part of TuDu.                                            *
  *                                                                        *
@@ -49,9 +49,11 @@ public:
 	void editDeadline(int line, Date& deadline, bool done);
 	bool editSched(Date& s);
 	void setPriority(int line, int& priority);
-	void setCategory(int line, string& category);
+	void setCategory(int line, ToDo& t);
 	void treeClear();
 	int treeLines();
+	bool searchText(string& pattern);
+	bool cmd(string& command);
 	void infoMsg(const char str[]);
 	void infoClear();
 	void infoPercent(int percent);
@@ -67,8 +69,11 @@ private:
 	Window *wschedule;
 	Config &config;
 	LineEditor lineEditor;
+	CategoryEditor categoryEditor;
 	DateEditor dateEditor;
 	PriorityEditor priorityEditor;
+	HistoryEditor searchEditor;
+	CmdEditor cmdEditor;
 	window_coor coor[NUM_WINDOWS];
 
 	void draw();
