@@ -1,6 +1,6 @@
 
 /**************************************************************************
- * Copyright (C) 2007-2008 Ruben Pollan Bella <meskio@amedias.org>        *
+ * Copyright (C) 2007-2009 Ruben Pollan Bella <meskio@sindominio.net>     *
  *                                                                        *
  *  This file is part of TuDu.                                            *
  *                                                                        *
@@ -25,9 +25,12 @@
 #include "config.h"
 
 #include <iostream>
+#include <string>
 #include <cstdio>
 #include <cstring>
+#include <errno.h>
 using namespace std;
+extern int errno;
 
 #ifndef SHARE_DIR
 #define SHARE_DIR "/usr/local/share/tudu"
@@ -37,7 +40,7 @@ using namespace std;
 #define ETC_DIR "/usr/local/etc"
 #endif
 
-#define VERSION "TuDu 0.4.1 (2008)"
+#define VERSION "TuDu 0.5.1 (2009)"
 #define WELCOME_FILE SHARE_DIR"/welcome.xml"
 #define CONFIG_FILE ETC_DIR"/tudurc"
 
@@ -51,14 +54,14 @@ using namespace std;
 
 #define version() \
 	cout << VERSION << endl; \
-	cout << "TuDu Copyright (C) 2007-2008 Ruben Pollan Bella <meskio@amedias.org>" << endl; \
+	cout << "TuDu Copyright (C) 2007-2009 Ruben Pollan Bella <meskio@sindominio.net>" << endl; \
 	cout << "TuDu comes with ABSOLUTELY NO WARRANTY; for details type `tudu -vv'" << endl; \
-	cout << "This is free software, and you are welcome to redistribute it" << endl; \
-	cout << "under certain conditions; type `tudu -vv' for details." << endl;
+	cout << "This is free software; you are welcome to redistribute it" << endl; \
+	cout << "under certain conditions. Type `tudu -vv' for details." << endl;
 
 #define copyright() \
 	cout << VERSION << endl; \
-	cout << "Copyright (C) 2007-2008 Ruben Pollan Bella <meskio@amedias.org>" << endl << endl; \
+	cout << "Copyright (C) 2007-2009 Ruben Pollan Bella <meskio@sindominio.net>" << endl << endl; \
  	cout << "TuDu is free software; you can redistribute it and/or modify" << endl; \
  	cout << "it under the terms of the GNU General Public License as published by" << endl; \
  	cout << "the Free Software Foundation; either version 3 of the License." << endl << endl; \
@@ -69,17 +72,39 @@ using namespace std;
  	cout << "You should have received a copy of the GNU General Public License" << endl; \
 	cout << "along with this program.  If not, see <http://www.gnu.org/licenses/>." << endl;
 
+/* return true for delete lock file and false for close tudu */
+bool lock_ask()
+{
+	cout << endl << "Lock file found" << endl;
+	cout << endl << "or" << endl;
+	cout << "    Another program may be editing the same file." << endl;
+	cout << "    If this is the case, be careful not to end up with two" << endl;
+	cout << "    different instances of the same file when making changes." << endl;
+	cout << "    Quit, or continue with caution." << endl;
+	cout << endl << "or" << endl;
+	cout << "    A tudu session for this file crashed." << endl;
+	cout << "    \"Edit anyway\" will destroy the lock file." << endl;
+	cout << endl << "[E]dit anyway, [Q]uit:";
+	string str;
+       	cin >> str;
+
+	if ((str[0] == 'E') || (str[0] == 'e'))
+		return true;
+	else
+		return false;
+}
+
 int main(int argc, char **argv, char *env[])
 {
 	int i;
-	char file_rc[128], file_xml[128];
+	char file_rc[128], file_xml[128], file_lock[133];
 
 	for (i = 0; strncmp(env[i],"HOME=",5); ++i);
 
 	Config config;
 	if (!config.load(CONFIG_FILE))
 	{
-		fprintf(stderr, "Err: Global config don't exist. The config should be %s\n", CONFIG_FILE);
+		fprintf(stderr, "Err: Global config does not exist. The config should be %s\n", CONFIG_FILE);
 		exit(1);
 	}
 	strncpy(file_rc,env[i]+5,119);
@@ -89,29 +114,46 @@ int main(int argc, char **argv, char *env[])
 	strncpy(file_xml,env[i]+5,117);
 	strcat(file_xml,"/.tudu.xml");
 
+	/*
+	 * Parse the comand line arguments
+	 */
 	for (i = 1; i < argc; ++i)
 	{
 		if (!strncmp("-f",argv[i],2))
 		{
 			++i;
-			strncpy(file_xml, argv[i], 127);
+			if (i < argc)
+				strncpy(file_xml, argv[i], 127);
+			else
+			{
+				usage();
+				return 0;
+			}
 		}
 		else if (!strncmp("-c",argv[i],2))
 		{
 			++i;
-			if (argv[i][0] != '/')
+			if (i < argc)
 			{
-				int j;
-				for (j = 0; strncmp(env[j],"PWD=",4); ++j);
-				strncpy(file_rc,env[j]+4,99);
-				strcat(file_rc,"/");
-				strncat(file_rc,argv[i],27);
+				if (argv[i][0] != '/')
+				{
+					int j;
+					for (j = 0; strncmp(env[j],"PWD=",4); ++j);
+					strncpy(file_rc,env[j]+4,99);
+					strcat(file_rc,"/");
+					strncat(file_rc,argv[i],27);
+				}
+				else
+				{
+					strncpy(file_rc, argv[i], 127);
+				}
+				config.load(file_rc);
 			}
 			else
 			{
-				strncpy(file_rc, argv[i], 127);
+				usage();
+				return 0;
 			}
-			config.load(file_rc);
 		}
 		else if (!strncmp("-vv",argv[i],3))
 		{
@@ -131,6 +173,43 @@ int main(int argc, char **argv, char *env[])
 		}
 	}
 
+	/*
+	 * Check and create the lock file
+	 */
+	strcpy(file_lock,file_xml);
+	for (i = strlen(file_lock); (file_lock[i] != '/') && (i > 0); i--);
+	if (file_lock[i] == '/') i++;
+	if (file_lock[i] != '.')
+	{
+		file_lock[i] = '.';
+		file_lock[i+1] = '\0';
+		strcat(file_lock, file_xml+i);
+	}
+	strcat(file_lock,"_lock");
+	// FIXME: it wont work with NFS
+	int lock;
+	lock = open(file_lock, O_CREAT|O_EXCL, 00666);
+	if (lock == -1)
+	{
+		if (errno == EEXIST)
+		{
+			if (!lock_ask()) exit(1);
+		}
+		/* no rights to write in this folder skip the problem */
+		else if (errno != EACCES)
+		{
+			fprintf(stderr, "Err: I can not create the lock file %s\n", file_lock);
+			exit(1);
+		}
+	}
+	else
+	{
+		close(lock);
+	}
+
+	/*
+	 * Load data
+	 */
 	ToDo node("");
 	iToDo it(node);
 	Sched sched;
@@ -141,14 +220,24 @@ int main(int argc, char **argv, char *env[])
 		/* welcome file don't exist */
 		if (!welcome.parse(node,sched))
 		{
-			fprintf(stderr, "Err: Welcome file don't exist. It should be %s\n", WELCOME_FILE);
+			fprintf(stderr, "Err: Welcome file does not exist. It should be %s\n", WELCOME_FILE);
+			unlink(file_lock);
 			exit(1);
 		}
 	}
 
+	/*
+	 * Load end start interface
+	 */
 	Writer w(file_xml,node);
 	Screen screen(config);
-	Interface in(screen,it,sched,config,w);
+	Cmd cmd;
+	Interface in(screen,it,sched,config,w,cmd);
 	in.main();
+
+	/*
+	 * Delete lock file
+	 */
+	unlink(file_lock);
 	return 0;
 }
